SystemOrganization addCategory: #'DBus-Tools'!

AbstractHierarchicalList subclass: #DBusExplorer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Tools'!

!DBusExplorer commentStamp: '<historical>' prior: 0!
Browse DBus services by their introspection data.
!

!DBusExplorer class methodsFor: 'class initialization' stamp: 'bf 6/17/2008 16:26'!
initialize
	TheWorldMenu registerOpenCommand: {'DBus Explorer'. {self. #open}}! !

!DBusExplorer class methodsFor: 'opening' stamp: 'bf 6/17/2008 16:24'!
open
	^self new inAWindow openInWorld
! !

!DBusExplorer class methodsFor: 'class initialization' stamp: 'bf 6/17/2008 16:28'!
unload
	TheWorldMenu unregisterOpenCommand: 'DBus Explorer'! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 7/7/2008 12:48'!
chooseProxyClass
	| proxyNames proxyName |
	proxyNames := (DBusCompiledProxy allSubclasses collect: [:cls | cls name]) asArray sort.
	proxyName := proxyNames at: (PopUpMenu withCaption: 'proxy class' chooseFrom: proxyNames) ifAbsent: [^ nil].
	^ Smalltalk classNamed: proxyName! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 3/3/2008 12:54'!
classTemplateFor: aDBusName
	| className |
	className := (aDBusName copyReplaceAll:  '.' with: ' ') toCamelCase capitalized.
	^'DBusCompiledProxy subclass: #', className,'
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''DBus-Proxies'''
! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 7/11/2008 20:10'!
compileDelegationMethods
	(DBusCompiledProxy classForName: currentSelection service andPath: currentSelection path)
		compileMethodsAndSignalsFrom: currentSelection proxy introspect

! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 7/11/2008 20:10'!
compileDelegationMethodsIn
	(self chooseProxyClass ifNil: [^ nil])
		compileMethodsAndSignalsFrom: currentSelection proxy introspect
! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 3/28/2008 13:04'!
createProxyClass
	| template class |
	template := FillInTheBlankMorph 
		request: 'Create DBus Proxy Class'
		initialAnswer: (self classTemplateFor: currentSelection service)
		centerAt: ActiveHand position
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true
		answerExtent: 600@180.
	template ifNil: [^self].
	class := Compiler evaluate: template.
	class class
		compile: 'dbusConnection\	^' withCRs, currentSelection connection printString classified: 'accessing';
		compile: 'dbusName\	^' withCRs, currentSelection service printString classified: 'accessing';
		compile: 'dbusPath\	^' withCRs, currentSelection path printString classified: 'accessing'! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 7/7/2008 12:50'!
genericMenu: aMenu 
	| hasMethods isMethod proxyClass |
	currentSelection
		ifNil: [aMenu add: '*nothing selected*' target: self selector: #yourself]
		ifNotNil: [
			proxyClass := DBusCompiledProxy classForName: currentSelection service andPath: currentSelection path.
			hasMethods := currentSelection contents anySatisfy: [:each | each hasContents not].
			isMethod := currentSelection hasContents not.
			isMethod	
				ifTrue: [aMenu add: 'invoke method' target: self selector: #invokeSelection]
				ifFalse: [aMenu add: 'inspect proxy' target: self selector: #inspectSelection].
			hasMethods ifTrue:
				[proxyClass
					ifNil: [aMenu add: 'create proxy class' target: self selector: #createProxyClass;
						add: 'compile all in ...' target: self selector: #compileDelegationMethodsIn]
					ifNotNil: [aMenu add: 'compile all in class ', proxyClass name target: self selector: #compileDelegationMethods]]].
	^aMenu! !

!DBusExplorer methodsFor: 'accessing' stamp: 'bf 6/16/2008 16:28'!
getList
	"Get all running DBus services by looking up registered names"

	^{DBus systemBus. DBus sessionBus}
		collect: [:each | DBusExplorerWrapper connection: each service: '' path: '']		

! !

!DBusExplorer methodsFor: 'opening' stamp: 'bf 2/21/2008 11:04'!
inAWindow
	| window  |

	window := (SystemWindow labelled: self class name) model: self.
	window 
		addMorph: self notInAWindow
		frame: (0@0 corner: 1@1).
     ^ window! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 3/28/2008 13:20'!
inspectSelection
	^currentSelection proxy inspect! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 6/16/2008 16:53'!
invokeSelection
	^currentSelection invokeMethod! !

!DBusExplorer methodsFor: 'menus' stamp: 'bf 6/17/2008 17:20'!
keyStroke: aCharacter
	({Character space. Character cr} includes: aCharacter)
		ifTrue: [self invokeSelection]! !

!DBusExplorer methodsFor: 'opening' stamp: 'bf 6/17/2008 17:17'!
notInAWindow
	| listMorph |

	(listMorph := SimpleHierarchicalListMorph 
		on: self
		list: #getList
		selected: #getCurrentSelection
		changeSelected: #noteNewSelection:
		menu: #genericMenu:
		keystroke: #keyStroke:).
	listMorph autoDeselect: false.
     ^ listMorph! !

ListItemWrapper subclass: #DBusExplorerWrapper
	instanceVariableNames: 'connection service path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Tools'!

!DBusExplorerWrapper commentStamp: 'bf 2/21/2008 12:47' prior: 0!
service:	'my.service'
path:	'/my/path interface.method()'!

!DBusExplorerWrapper class methodsFor: 'instance creation' stamp: 'bf 3/28/2008 13:40'!
connection: aConnection service: serviceName path: pathString
	^self new connection: aConnection service: serviceName path: pathString! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 6/16/2008 16:34'!
asString
	"display only the 'interesting' part of the path"

	^self hasContents
		ifTrue: [path size <= 1
			ifTrue: [service
				ifEmpty: [connection asString]
				ifNotEmpty: [service]]
			ifFalse: [path copyAfterLast: $/]]
		ifFalse: [path copyAfter: Character space]! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 3/28/2008 13:38'!
connection
	^connection! !

!DBusExplorerWrapper methodsFor: 'initialize-release' stamp: 'bf 3/28/2008 13:38'!
connection: aConnection service: serviceName path: pathString
	connection := aConnection.
	service := serviceName.
	path := pathString.! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 6/16/2008 16:37'!
contents
	"introspect the current path, answer children (subnodes of current path) and leafs (methods, signals)"

	| node children leafs in out |
	self hasContents ifFalse: [^#()].
	service isEmpty ifTrue: [
		^((connection getObject:  'org.freedesktop.DBus' path: '/') listNames
			reject: [:each | each first = $:])
				collect: [:each | DBusExplorerWrapper connection: connection service: each path: '/']].
	node := (XMLDOMParser parseDocumentFrom: self proxy introspect readStream) firstTagNamed: #node .
	children := (node elements select: [:each | each tag = #node])
		collect: [:each | DBusExplorerWrapper
			connection: connection
			service: service
			path: (path='/' ifTrue: [path] ifFalse: [path, '/']), (each attributeAt: 'name')].
	leafs := Array streamContents: [:strm |
		node tagsNamed: #interface do: [:interface |
			interface elements do: [:each |
				(#(method signal) includes: each tag) ifTrue: [
				in := self printParameters: 'in' from: each.
				out := each tag = #signal
					ifTrue: ['SIGNAL']
					ifFalse: [self printParameters: 'out' from: each].
				strm nextPut: (DBusExplorerWrapper
					connection: connection
					service: service
					path: path, ' ', (interface attributeAt: 'name'), '.', (each attributeAt: 'name'), in, ' => ', out)]]]].

	^children, leafs! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 3/3/2008 18:06'!
fullSelector
	^(path copyAfter: Character space)
		copyUpTo: $(! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 2/21/2008 12:44'!
hasContents
	"If I am a leaf (method, signal) then my path looks like '/my/path interface.method()'."

	^(path includes: Character space) not! !

!DBusExplorerWrapper methodsFor: 'private' stamp: 'bf 6/17/2008 19:44'!
invokeMethod
	| args result argName argSig argValue |
	args := ((path copyAfter: $() copyUpTo: $)) findTokens: $,.
	args := args collect: [:each |
			argName := each copyUpTo: $:.
			argSig := each copyAfter: $:.
			argValue := FillInTheBlank request:
				argName, 	' (', (DBusArgument nameOfSignature: argSig), ')'.
			argValue ifNil: [^self].
			argValue := argSig = 's'
				ifTrue: [argValue]
				ifFalse: [Compiler evaluate: argValue].
			DBusArgument value: argValue signature: argSig].
	"execute in background since the dbus call blocks which might interfere with a synchronous server in this same image"
	[
		result := self proxy dbusPerform: self fullSelector withArguments: args.
		result ifNotNil: [WorldState addDeferredUIMessage: [result inspect]]
	] fork! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 3/3/2008 18:04'!
path
	^path copyUpTo: Character space! !

!DBusExplorerWrapper methodsFor: 'private' stamp: 'bf 3/27/2008 13:25'!
printParameters: direction from: aNode
	"given introspection data for a method or signal node, format a parameter list like (name:type, name:type)"

	^String streamContents: [:strm |
		strm nextPut: $(.
		(aNode elements select: [:each | each tag = #arg and: [(each attributeAt: 'direction' ifAbsent: ['in']) = direction]])
			do: [:arg | strm nextPutAll: (arg attributeAt: 'name' ifAbsent: ['']), ':', (arg attributeAt: 'type')]
			separatedBy: [strm nextPutAll: ', '].
		strm nextPut: $)].! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 3/28/2008 13:52'!
proxy
	^self connection getObject: self service path: self path! !

!DBusExplorerWrapper methodsFor: 'accessing' stamp: 'bf 3/3/2008 12:49'!
service
	^service! !
DBusExplorer initialize!
